@@ -153,7 +153,6 @@ struct crl_cache_entry_s
 struct crl_cache_s 
 {
   crl_cache_entry_t entries;
-  unsigned int cdb_open_count; /* Currently open cache files. */
 };
 
 typedef struct crl_cache_s *crl_cache_t;
@@ -945,12 +944,14 @@ check_dbfile (const char *fname, const char *md5hexvalue)
 
 /* Open the cache file for ENTRY.  This function implements a caching
    strategy and might close unused cache files. It is required to use
-   unlock_db_file after the using the file. */ 
+   unlock_db_file after using the file. */ 
 static struct cdb *
 lock_db_file (crl_cache_t cache, crl_cache_entry_t entry)
 {
   char *fname;
   int fd;
+  int open_count;
+  crl_cache_entry_t e;
 
   if (entry->cdb)
     {
@@ -958,12 +959,20 @@ lock_db_file (crl_cache_t cache, crl_cache_entry_t entry)
       return entry->cdb;
     }
 
+  for (open_count = 0, e = cache->entries; e; e = e->next)
+    {
+      if (e->cdb)
+        open_count++;
+/*       log_debug ("CACHE: cdb=%p use_count=%u lru_count=%u\n", */
+/*                  e->cdb,e->cdb_use_count,e->cdb_lru_count); */
+    }
+
   /* If there are too many file open, find the least recent used DB
      file and close it.  Note that for Pth thread safeness we need to
      use a loop here. */
-  while (cache->cdb_open_count >= MAX_OPEN_DB_FILES )
+  while (open_count >= MAX_OPEN_DB_FILES )
     {
-      crl_cache_entry_t e, last_e = NULL;
+      crl_cache_entry_t last_e = NULL;
       unsigned int last_lru = (unsigned int)(-1);
 
       for (e = cache->entries; e; e = e->next)
@@ -978,14 +987,15 @@ lock_db_file (crl_cache_t cache, crl_cache_entry_t entry)
           return NULL;
         }
 
+/*       log_debug ("CACHE: closing file at cdb=%p\n", last_e->cdb); */
+
       fd = cdb_fileno (last_e->cdb);
       cdb_free (last_e->cdb);
       xfree (last_e->cdb);
       last_e->cdb = NULL;
       if (close (fd))
         log_error (_("error closing cache file: %s\n"), strerror(errno));
-      assert (cache->cdb_open_count);
-      cache->cdb_open_count--;
+      open_count--;
     }
 
 
@@ -1031,7 +1041,6 @@ lock_db_file (crl_cache_t cache, crl_cache_entry_t entry)
 
   entry->cdb_use_count = 1;
   entry->cdb_lru_count = 0;
-  cache->cdb_open_count++;
 
   return entry->cdb;
 }
@@ -1074,7 +1083,7 @@ unlock_db_file (crl_cache_t cache, crl_cache_entry_t entry)
 static crl_cache_entry_t 
 find_entry (crl_cache_entry_t first, const char *issuer_hash)
 {
-  while (first && !first->deleted && strcmp (issuer_hash, first->issuer_hash) )
+  while (first && (first->deleted || strcmp (issuer_hash, first->issuer_hash)))
     first = first->next;
   return first;    
 }
@@ -1365,92 +1374,6 @@ crl_cache_cert_isvalid (ctrl_t ctrl, ksba_cert_t cert,
 }  
 
 
-/* Return the certificate for NAME or NULL if it can't be found.  Note
-   that we will only return the first matching certificate. */
-static ksba_cert_t 
-get_crlissuer_cert (ctrl_t ctrl, const char *name)
-{
-  gpg_error_t err;
-  ksba_cert_t cert = NULL;
-  cert_fetch_context_t context;
-
-  /* First check whether it has already been cached.  */
-  cert = get_cert_bysubject (name, 0); 
-  if (cert)
-    return cert; /* Done. */
-    
-  /* Ask back to the service requester to return the certificate.
-     This is because we can assume that he already used the
-     certificate while checking for the CRL. */
-  cert = get_cert_local (ctrl, name);
-  if (cert)
-    return cert; /* Done. */
-
-  /* If this fails do a regular certificate lookup. */
-  if (DBG_LOOKUP)
-    log_debug ("certificate for CRL issuer not returned by caller"
-               " - doing lookup\n");
-
-  err = ca_cert_fetch (ctrl, &context, name);
-  if (err)
-    {
-      log_error (_("error fetching certificate for CRL issuer: %s\n"),
-                 gpg_strerror (err));
-      return NULL;
-    }
-
-  err = fetch_next_ksba_cert (context, &cert);
-  if (err)
-    log_error (_("invalid CRL issuer certificate: %s\n"), gpg_strerror (err) );
-  end_cert_fetch (context);
-
-  if (!err)
-    cache_cert (cert);
-
-  return cert;
-}
-
-
-/* Return the certificate from the issuer DN NAME and the S/N SN.
-   First tries the local chache, then asks the caller and then tries
-   an ldap search. */
-static ksba_cert_t 
-get_crlissuer_cert_bysn (ctrl_t ctrl, const char *name, ksba_sexp_t sn)
-{
-  ksba_cert_t cert;
-  char *snbuf, *buf;
-
-  /* First check whether it has already been cached.  */
-  cert = get_cert_bysn (name, sn);
-  if (cert)
-    return cert; /* Done. */
-
-  /* Ask back to the service requester to return the certificate.
-     This is because we can assume that he already used the
-     certificate while checking for the CRL. */
-  snbuf = serial_hex (sn);
-  if (!snbuf)
-    snbuf = "";
-  buf = xtrymalloc (1 + strlen (snbuf) + 1 + strlen (name) + 1 );
-  if (!buf)
-    {
-      xfree (snbuf);
-      return NULL;
-    }
-  strcpy (stpcpy (stpcpy (stpcpy (buf, "#"), snbuf), "/"), name);
-  xfree (snbuf);
-  cert = get_cert_local (ctrl, buf);
-  xfree (buf);
-  if (cert)
-    {
-      cache_cert (cert);
-      return cert; /* Done. */
-    }
-
-  return find_cert_bysn (ctrl, name, sn);
-}
-
-
 /* Prepare a hash context for the signature verification.  Input is
    the CRL and the output is the hash context MD as well as the uses
    algorithm identifier ALGO. */
@@ -1576,7 +1499,7 @@ finish_sig_check (ksba_crl_t crl, gcry_md_hd_t md, int algo,
    required to retrieve certificates using the general dirmngr
    callback service.  R_CRLISSUER returns an allocated string with the
    crl-issuer DN, THIS_UPDATE and NEXT_UPDATE are filled with the
-   correspondind data from the CRL.  Note that these values might get
+   corresponding data from the CRL.  Note that these values might get
    set even if the CRL processing fails at a later step; thus the
    caller should free *R_ISSUER even if the function returns with an
    error.
@@ -1674,6 +1597,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
             char *crlissuer;
             ksba_name_t authid;
             ksba_sexp_t authidsn;
+            ksba_sexp_t keyid;
 
             /* We need to look for the issuer only after having read
                all items.  The issuer itselfs comes before the items
@@ -1690,7 +1614,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
             *r_crlissuer = crlissuer; /* (Do it here so we don't need
                                          to free it later) */
 
-            if (!ksba_crl_get_auth_key_id (crl, NULL, &authid, &authidsn))
+            if (!ksba_crl_get_auth_key_id (crl, &keyid, &authid, &authidsn))
               {
                 const char *s;
 
@@ -1700,20 +1624,36 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
                 
                 s = ksba_name_enum (authid, 0);
                 if (s && *authidsn)
-                  crlissuer_cert = get_crlissuer_cert_bysn (ctrl, s, authidsn);
+                  crlissuer_cert = find_cert_bysn (ctrl, s, authidsn);
+                if (!crlissuer_cert && keyid)
+                  crlissuer_cert = find_cert_bysubject (ctrl,
+                                                        crlissuer, keyid);
+
                 if (!crlissuer_cert)
                   {
-                    log_info ("CRL issuer certificate (#");
-                    dump_serial (authidsn);
-                    log_printf ("/");
-                    dump_string (s);
-                    log_printf (") not found\n");
+                    log_info ("CRL issuer certificate ");
+                    if (keyid)
+                      {
+                        log_printf ("{");
+                        dump_serial (keyid);
+                        log_printf ("} ");
+                      }
+                    if (authidsn)
+                      {
+                        log_printf ("(#");
+                        dump_serial (authidsn);
+                        log_printf ("/");
+                        dump_string (s);
+                        log_printf (") ");
+                      }
+                    log_printf ("not found\n");
                   }
                 ksba_name_release (authid);
                 xfree (authidsn);
+                xfree (keyid);
               }
             else
-              crlissuer_cert = get_crlissuer_cert (ctrl, crlissuer);
+              crlissuer_cert = find_cert_bysubject (ctrl, crlissuer, NULL);
             err = 0;
             if (!crlissuer_cert)
               {
@@ -1729,7 +1669,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
                 goto failure;
               }
             err = validate_cert_chain (ctrl, crlissuer_cert,
-                                       NULL, VALIDATE_MODE_CRL);
+                                       NULL, VALIDATE_MODE_CRL_RECURSIVE);
             if (err)
               {
                 log_error (_("error checking validity of CRL "
@@ -2252,6 +2192,9 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
                                                 &distpoint,
                                                 &issuername, NULL )))
     {
+      int name_seq; 
+      gpg_error_t last_err = 0;
+
       if (!distpoint && !issuername)
         {
           if (DBG_X509)
@@ -2261,52 +2204,71 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
                     suitable CRL. */
         }
 
-      /* Get the URIs. */
-      xfree (distpoint_uri); distpoint_uri = NULL;
       xfree (issuername_uri); issuername_uri = NULL;
-      distpoint_uri = ksba_name_get_uri (distpoint, 0); 
-      issuername_uri = ksba_name_get_uri (issuername, 0); 
+
+      /* Get the URIs.  We do this in a loop to iterate over all names
+         in the crlDP. */
+      for (name_seq=0; ksba_name_enum (distpoint, name_seq); name_seq++)
+        {
+          xfree (distpoint_uri); distpoint_uri = NULL;
+          distpoint_uri = ksba_name_get_uri (distpoint, name_seq);
+          if (!distpoint_uri)
+            continue;
+          
+          if (!strncmp (distpoint_uri, "ldap:", 5)
+              || !strncmp (distpoint_uri, "ldaps:", 6))
+            {
+              if (opt.ignore_ldap_dp)
+                continue;
+            }
+          else if (!strncmp (distpoint_uri, "http:", 5)
+                   || !strncmp (distpoint_uri, "https:", 6))
+            {
+              if (opt.ignore_http_dp)
+                continue;
+            }
+          else
+            continue; /* Skip unknown schemes. */
+          
+          any_dist_point = 1;
+          
+          if (DBG_X509)
+            log_debug ("fetching CRL from `%s'\n", distpoint_uri);
+          err = crl_fetch (ctrl, distpoint_uri, &reader);
+          if (err)
+            {
+              log_error (_("crl_fetch via DP failed: %s\n"),
+                         gpg_strerror (err));
+              last_err = err;
+              continue; /* with the next name. */
+            }
+          
+          if (DBG_X509)
+            log_debug ("inserting CRL\n");
+          err = crl_cache_insert (ctrl, distpoint_uri, reader); 
+          if (err)
+            {
+              log_error (_("crl_cache_insert via DP failed: %s\n"),
+                         gpg_strerror (err));
+              last_err = err;
+              continue; /* with the next name. */
+            }
+          last_err = 0;
+          break; /* Ready. */
+        }
+      if (last_err)
+        {
+          err = last_err;
+          goto leave;
+        }
+      
       ksba_name_release (distpoint); distpoint = NULL;
+
+      /* We don't do anything with issuername_uri yet but we keep the
+         code for documentation. */
+      issuername_uri =  ksba_name_get_uri (issuername, 0); 
       ksba_name_release (issuername); issuername = NULL;
 
-      if (!distpoint_uri)
-        continue;
-
-      if (!strncmp (distpoint_uri, "ldap:", 5)
-          || !strncmp (distpoint_uri, "ldaps:", 6))
-        {
-          if (opt.ignore_ldap_dp)
-            continue;
-        }
-      else if (!strncmp (distpoint_uri, "http:", 5)
-               || !strncmp (distpoint_uri, "https:", 6))
-        {
-          if (opt.ignore_http_dp)
-            continue;
-        }
-      else
-        continue; /* Skip unknown schemes. */
-
-      any_dist_point = 1;
-      
-      if (DBG_X509)
-        log_debug ("fetching CRL from `%s'\n", distpoint_uri);
-      err = crl_fetch (ctrl, distpoint_uri, &reader);
-      if (err)
-        {
-          log_error (_("crl_fetch via DP failed: %s\n"), gpg_strerror (err));
-          goto leave;
-        }
-      
-      if (DBG_X509)
-        log_debug ("inserting CRL\n");
-      err = crl_cache_insert (ctrl, distpoint_uri, reader); 
-      if (err)
-        {
-          log_error (_("crl_cache_insert via DP failed: %s\n"),
-                     gpg_strerror (err));
-          goto leave;
-        }
     }
   if (gpg_err_code (err) == GPG_ERR_EOF)
     err = 0;
