@@ -1,6 +1,6 @@
 /* dirmngr.c - LDAP access
  *	Copyright (C) 2002 Klarälvdalens Datakonsult AB
- *      Copyright (C) 2003, 2004 g10 Code GmbH
+ *      Copyright (C) 2003, 2004, 2006 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -90,6 +90,8 @@ enum cmd_and_opt_values
   oLDAPAddServers,
   oOCSPResponder,
   oOCSPSigner,
+  oOCSPMaxClockSkew,
+  oOCSPCurrentPeriod,
   oMaxReplies,
   oFakedSystemTime,
   oForce,
@@ -156,6 +158,8 @@ static ARGPARSE_OPTS opts[] = {
 
   { oOCSPResponder, "ocsp-responder", 2, N_("|URL|use OCSP responder at URL")},
   { oOCSPSigner, "ocsp-signer", 2, N_("|FPR|OCSP response signed by FPR")}, 
+  { oOCSPMaxClockSkew, "ocsp-max-clock-skew", 1, "@" },
+  { oOCSPCurrentPeriod, "ocsp-current-period", 1, "@" },
 
   { oMaxReplies, "max-replies", 1,
     N_("|N|do not return more than N items in one query")},
@@ -217,7 +221,7 @@ my_strusage( int level )
       case 11: p = "dirmngr";
 	break;
       case 13: p = VERSION; break;
-      case 14: p = "Copyright (C) 2004 g10 Code GmbH"; break;
+      case 14: p = "Copyright (C) 2006 g10 Code GmbH"; break;
       case 17: p = PRINTABLE_OS_NAME; break;
       case 19: p =
 	    _("Please report bugs to <gpa-dev@gnupg.org>.\n");
@@ -401,6 +405,8 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
       opt.allow_ocsp = 0;
       opt.ocsp_responder = NULL;
       opt.ocsp_signer = NULL; 
+      opt.ocsp_max_clock_skew = 10 * 60;      /* 10 minutes.  */
+      opt.ocsp_current_period = 3 * 60 * 60;  /* 3 hours. */
       opt.max_replies = DEFAULT_MAX_REPLIES;
       return 1;
     }
@@ -445,6 +451,8 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
     case oAllowOCSP: opt.allow_ocsp = 1; break;
     case oOCSPResponder: opt.ocsp_responder = pargs->r.ret_str; break;
     case oOCSPSigner:    opt.ocsp_signer = pargs->r.ret_str; break;
+    case oOCSPMaxClockSkew: opt.ocsp_max_clock_skew = pargs->r.ret_int; break;
+    case oOCSPCurrentPeriod: opt.ocsp_current_period = pargs->r.ret_int; break;
 
     case oMaxReplies: opt.max_replies = pargs->r.ret_int; break;
 
@@ -541,7 +549,7 @@ main (int argc, char **argv )
   /* Reset rereadable options to default values. */
   parse_rereadable_options (NULL, 0); 
 
-  /* LDAP defaults */
+  /* LDAP defaults.  */
   opt.add_new_ldapservers = 0;
   opt.ldaptimeout = DEFAULT_LDAP_TIMEOUT;
 
